########################################################################
#
# save.prediction.file()
#
# This function should be used to save your predictions on the
# comp.test test set for Problem R-5.  The function saves your name
# (or a pseudonym of your choice), a short description of the method
# you used, and a vector of predictions on the test examples in
# comp.test to a file.
#
#
# Input parameters:
#
#    name        --  your name (or a pseudonym) as a string
#    description --  a one-sentence description (as a string) of the
#                     approach you used to generate these predictions
#    predictions --  a numeric vector of predictions, with one prediction
#                     for each test example in comp.test
#    filename    --  the name of the binary file in which to save the data
#                     (please use the default name "problemR5-preds.Rdata"
#                     when submitting)
#
########################################################################
save.prediction.file <- function(name, description, predictions,
                                 filename="problemR5-preds.Rdata") {
  n <- dim(comp.test)[1]
  if (!is.vector(predictions) || !is.numeric(predictions)
      || length(predictions) != n) {
    print("Error: 'predictions' must be a numeric vector of same length as comp.test", quote=FALSE);
    return;
  }
  if (!is.character(name)) {
    print("Error: 'name' must be a character string", quote=FALSE);
    return;
  }
  if (!is.character(description)) {
    print("Error: 'description' must be a character string", quote=FALSE);
    return;
  }

  save(name, description, predictions, file=filename)
}



########################################################################
# load all data -- this will happen automatically each time you type
#                  source("helper.R")
########################################################################

house.all <- read.table("house.dat",header=TRUE)
house.train <- house.all[1:500,]
house.test <- house.all[-(1:500),]

abalone.all <- read.table("abalone.dat",header=TRUE)
abalone.train <- abalone.all[1:500,]
abalone.test <- abalone.all[-(1:500),]

comp.train <- read.table("comp-activ.train",header=TRUE)
comp.test <- read.table("comp-activ.test",header=TRUE)
