########################################################################
#
# ridge()
#
# This is the template for a function that runs ridge regression on
# the given training set, and then evaluates the result on both
# training and test data.  It is assumed that the goal is to predict
# the value of the last column of the given training and test data
# frames using all of the other columns.
#
# Prior to running ridge regression, your code should center and scale
# the data, and should always include an intercept term.  (See the
# assignment instructions for more details.)  The weight vector that
# your code returns should be with respect to this centered and scaled
# data.
#
# (You may modify this function to return *additional* values in the
# returned list.  You also may modify this function (or the others
# below) to take additional *optional* input parameters, provided that
# the default values of these additional parameters cause the function
# to behave exactly like this one that we are providing.)
#
#
# Input parameters:
#
#    train.set  --  a data frame containing the training data
#    test.set   --  a data frame containing the test data
#    lambda     --  the regularization parameter
#
#
# Returns a list with the following values:
#
#    w0         -- the computed weight value for the intercept term
#    w          -- vector of computed weights for all other terms
#    train.pred -- vector of predictions on the training set
#    test.pred  -- vector of predictions on the test set
#    train.rmse -- RMSE on the training set
#    test.rmse  -- RMSE on the test set
#
########################################################################
ridge <- function(train.set, test.set, lambda) {

  # your code here

  # return the following list (with all fields filled in):
  list(w0 =   ,
       w =    ,
       train.pred=   ,
       test.pred=   ,
       train.rmse=   ,
       test.rmse=   ,
}



########################################################################
#
# run.co2()
#
# This is the template for a function that should run ridge() on the
# built-in co2 dataset using a basis including polynomial functions
# (t, t^2, t^3, etc., where t is time in years) and/or trigonometric
# functions (cos(2*pi*t) and sin(2*pi*t)).  The ridge regression model
# should be trained on data before cut.year, and tested on data during
# or after cut.year.
#
# For numerical stability, you may need to scale the polynomial basis
# functions.  Thus, rather than using t^k, you can use c*t^k where c
# is a constant chosen to prevent numerical overflow or underflow,
# such as c=1/M^k, where M is the largest value of t over both
# training and testing examples.  (Note that whatever constant you
# choose will get "washed out" when the data is rescaled by ridge().)
#
#
# Input parameters:
#
#    cut.year   -- cut-off year
#    inc.sin    -- TRUE if sin(2*pi*t) should be included in the basis
#    inc.cos    -- TRUE if cos(2*pi*t) should be included in the basis
#    polylim    -- limit on degree of polynomial terms to be included in basis
#                  (thus, if polylim=k, then the basis will include
#                   t, t^2, ... , t^k)  
#    ...        -- other options to ridge(), such as lambda
#
#
# Returns:
#    result of running ridge() on the generated training and test sets 
#
########################################################################
run.co2 <- function(cut.year=1975, inc.sin=FALSE, inc.cos=FALSE, polylim=1, ...) {

  # your code here

  # returns result of runnning ridge():
  ridge(   )  # fill in
}



########################################################################
#
# cross.val()
#
# This is the template for a function that should run k-fold cross
# validation on the provided dataset, returning the test RMSE averaged
# over k runs of ridge().  Specifically, your code should divide the
# given dataset into k disjoint, contiguous blocks of roughly equal
# size, and should then run ridge() k times: on the i-th iteration use
# block i as a test set after training on all of blocks except block
# i.
#
# Your code should *not* randomly permute the provided dataset before
# breaking it into blocks, but should rather use the given ordering of
# the examples.  This way, you can run cross validation using multiple
# settings of lambda, each time using the same division into blocks.
# However, in conducting your experiments, you *should* first put the
# examples in random order, and then use that same order for testing
# several different settings of lambda.
#
#
# Input parameters:
#
#    data       -- a data frame containing the provided dataset
#    folds      -- number of folds  
#    ...        -- other parameters to ridge(), such as lambda
#
#
# Returns:
#    test RMSE averaged over the k runs of ridge()
#
########################################################################
cross.val <- function(data, folds=10, ...) {

  # your code here

}
